import { RollStats } from "./rollstats"
import { Outcomes } from "./asphodice"

let rollstats = new RollStats();                                                                                    

function getResults():void {
	console.log("getting results");
	$("#results").empty();
	rollstats.doRolls();
	$("#results").append(`<div class="card my-4">`);
	let resultsCard = $("#results").find("div.card");
	resultsCard.append(`<div class="card-body">`);
	let resultsCardBody = $("#results").find("div.card-body");
	resultsCardBody.append(`<h2 class="card-title">Asphodice Results</h2>`);
	resultsCardBody.append(`<h3 class="card-subtitle my-1">Rerolls</h3>`);
	resultsCardBody.append(`<div class="row my-3 mx-auto">
			       <div class="col alert alert-success"><strong>Rerolled:</strong> ${rollstats.rerollCounts.true} (${(rollstats.rerollCounts.true / rollstats.numRolls * 100).toFixed(2)} %)</div>
				<div class="col alert alert-warning"><strong>Not rerolled:</strong> ${rollstats.rerollCounts.false} (${(rollstats.rerollCounts.false / rollstats.numRolls * 100).toFixed(2)} %)</div>
			     </div>`);

	resultsCardBody.append(`<h3 class="card-subtitle my-1">Outcome Balances</h3>`);
	resultsCardBody.append(`<table class="table table-striped" id="balanceTable">
			     <thead><th>Balance</th> <th>Count</th> <th>Percentage</th></thead>
			     <tbody></tbody>
			     </table>`);

	resultsCardBody.append(`<h3 class="card-subtitle my-1">Outcomes</h3>`);
	resultsCardBody.append(`<table class="table table-striped" id="outcomeTable">
			     <thead><th>Outcome</th> <th>Count</th> <th>Percentage</th></thead>
			     <tbody></tbody>
			     </table>`);

	let bc = rollstats.balanceCounts;
	let keys = Object.keys(bc);
	keys.sort(function(a: string, b: string){return Number(a) - Number(b)});
	for (var i = 0; i < keys.length; i++) {
		let tb = $("#balanceTable").find("tbody");
		tb.append(`<tr>	<td>${keys[i]}</td>
			  	<td>${rollstats.balanceCounts[keys[i]]}</td>
				<td>${(rollstats.balanceCounts[keys[i]] / rollstats.numRolls * 100).toFixed(2)}</td>
			</tr>`);
	}

	let oc = rollstats.outcomeCounts;
	let okeys = Object.keys(oc);
	for (var i = 0; i < okeys.length; i++) {
		let tb = $("#outcomeTable").find("tbody");
		let outcome: Outcomes = okeys[i];
		let outcomeCount = rollstats.outcomeCounts[outcome];
		let outcomePercent = (outcomeCount / rollstats.numRolls * 100).toFixed(2);
		tb.append(`<tr> <td>${outcome}</td>
			  <td>${outcomeCount}</td>
			  <td>${outcomePercent}</td>
			  </tr>`);
	}

	// Charts
	resultsCardBody.append(`<h3 class="card-subtitle my-2">Charts</h3>`);
	resultsCardBody.append(`<h4 class="card-subtitle my-1">Outcome Balances</h4>`);
	resultsCardBody.append(`<canvas id="balanceChart"></canvas>`);
	let balanceCanvas = $("#balanceChart");
	let balanceChart = new Chart(balanceCanvas, {
		type: "bar",
		data: {
			labels: Object.keys(rollstats.balanceCounts),
			datasets: [{
				label: "Outcome Balance Counts",
				data: Object.values(rollstats.balanceCounts),
				backgroundColor: palette("tol", Object.values(rollstats.balanceCounts).length).map(function(hex) { return `#${String(hex)}`; }),
			}],
		},
		options: {
			title: {
				text: "Balance Count Chart",
				display: false,
			},
			scales: {
				yAxes: [{
					ticks: {
						beginAtZero: true,
					}
				}]
			}
		}
	});

	resultsCardBody.append(`<h4 class="card-subtitle my-1">Outcomes</h4>`);
	resultsCardBody.append(`<canvas id="outcomesChart"></canvas>`);
	let outcomesCanvas = $("#outcomesChart");
	let outcomesChart = new Chart(outcomesCanvas, {
		type: "bar",
		data: {
			labels: Object.keys(rollstats.outcomeCounts),
			datasets: [{
				label: "Outcome Counts",
				data: Object.values(rollstats.outcomeCounts),
				backgroundColor: palette("tol", Object.values(rollstats.outcomeCounts).length).map(function(hex) { return `#${String(hex)}`; }),
			}],
		},
		options: {
			title: {
				text: "Balance Count Chart",
				display: false,
			},
			scales: {
				yAxes: [{
					ticks: {
						beginAtZero: true,
					}
				}]
			}
		}
	});

	resultsCardBody.append(`<h4 class="card-subtitle my-1">Rerolls</h4>`);
	resultsCardBody.append(`<canvas id="rerollsChart"></canvas>`);
	let rerollsCanvas = $("#rerollsChart");
	let rerollsChart = new Chart(rerollsCanvas, {
		type: "doughnut",
		data: {
			labels: Object.keys(rollstats.rerollCounts),
			datasets: [{
				label: "Reroll Counts",
				data: Object.values(rollstats.rerollCounts),
				backgroundColor: palette("tol", Object.values(rollstats.rerollCounts).length).map(function(hex) { return `#${String(hex)}`; }),
			}],
		},
		options: {
			title: {
				text: "Rerolls Chart",
				display: false,
			},
		}
	});
}


function setupHandlers(): void {
	console.log("adding event handler");
	$("#mainRoll").click(getResults);
}

document.addEventListener("DOMContentLoaded", setupHandlers);
